#include "ComConnection.h"
#include "ComConnectionException.h"
#include <iostream>

ComConnection::ComConnection(char *portName, int baudrate)
{
	this->connected = false;
	this->portName = portName;
	this->baudrate = baudrate;
	this->bytesize = 8;
	this->stopbits = 1;
	this->parity = 0;
}

ComConnection::ComConnection(char *portName, int baudrate, int stopbits, int bytesize, int parity)
{
	this->connected = false;
	this->portName = portName;
	this->baudrate = baudrate;
	this->bytesize = bytesize;
	this->stopbits = stopbits;
	this->parity = parity;
}

ComConnection::~ComConnection() {
	this->disconect();
}
 
void ComConnection::connect() {
	// Creating the GENERIC_READ | WRITE handler
	this->handler = CreateFileA(static_cast<LPCSTR>(portName),
		GENERIC_READ | GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL);
	// Throw exceptions in case value errors
	if (this->handler == INVALID_HANDLE_VALUE) {
		if (GetLastError() == ERROR_FILE_NOT_FOUND) {
			throw ComConnectionException("Handle was not attached. Reason: port not available");
		}
		else {
			throw ComConnectionException("Error during during ComConnection creation");
		}
	}
	else {
		// Setup serial parameters from to connection. Throw exceptions if failed.
		DCB dcbSerialParameters = { 0 };
		if (!GetCommState(this->handler, &dcbSerialParameters)) {
			throw ComConnectionException("Failed to get current serial parameters");
		}
		else {
			dcbSerialParameters.BaudRate = this->baudrate;
			dcbSerialParameters.ByteSize = this->bytesize;
			dcbSerialParameters.StopBits = this->stopbits;
			dcbSerialParameters.Parity = this->parity;
			dcbSerialParameters.fDtrControl = DTR_CONTROL_ENABLE;

			if (!SetCommState(handler, &dcbSerialParameters)) {
				throw ComConnectionException("Could not set serial port parameters");
			}
			else {
				this->connected = true;
				PurgeComm(this->handler, PURGE_RXCLEAR | PURGE_TXCLEAR);
				Sleep(WAIT_TIME);
			}
		}
	}
}

void ComConnection::disconect() {
	if (this->connected) {
		this->connected = false;
		CloseHandle(this->handler);
	}
}

int ComConnection::readSerialPort(char *buffer, unsigned int buf_size)
{
	DWORD bytesRead;
	unsigned int toRead = 0;

	ClearCommError(this->handler, &this->errors, &this->status);

	if (this->status.cbInQue > 0) {
		if (this->status.cbInQue > buf_size) {
			toRead = buf_size;
		}
		else toRead = this->status.cbInQue;
	}

	memset(buffer, 0, buf_size);

	if (ReadFile(this->handler, buffer, toRead, &bytesRead, NULL)) return bytesRead;

	return 0;
}

bool ComConnection::writeSerialPort(const std::string& command, unsigned int buf_size)
{
	char *buffer = const_cast<char*>(command.c_str());
	DWORD bytesSend;
	if (!WriteFile(this->handler, (void*)buffer, buf_size, &bytesSend, 0)) {
		ClearCommError(this->handler, &this->errors, &this->status);
		return false;
	}
	return true;
}

bool ComConnection::isConnected() const
{
	return this->connected;
}
 
char* ComConnection::getDeviceName() const {
	return this->portName;
}